@@ -49,6 +49,15 @@ ogrInfo <- function(dsn, layer, encoding=NULL, input_field_name_encoding=NULL,
 
   if (swapAxisOrder) ogrinfo[[5]] <- ogrinfo[[5]][c(2,1,4,3)]
 
+  u_eType <- u_with_z <- null_geometries <- NULL
+  deleted_geometries <- NULL
+  retain <- NULL
+  have_features <- NULL
+  all_NULL <- FALSE
+  keepGeoms <- NULL
+
+  if (!is.na(ogrinfo[[1]])) {
+
   fids <- ogrFIDs(dsn=dsn, layer=layer)
   nrows_i <- attr(fids, "i")
   have_features <- nrows_i > 0
@@ -64,8 +73,6 @@ ogrInfo <- function(dsn, layer, encoding=NULL, input_field_name_encoding=NULL,
      retain <- NULL
   }
 #  attributes(fids) <- NULL
-  u_eType <- u_with_z <- null_geometries <- NULL
-
   if (have_features) {
     eTypes <- .Call("R_OGR_types",as.character(dsn), as.character(layer),
       PACKAGE = "rgdal")
@@ -80,29 +87,31 @@ ogrInfo <- function(dsn, layer, encoding=NULL, input_field_name_encoding=NULL,
     }
     null_geometries <- NULL
     if (any(isNULL)) {
+      all_NULL <- (sum(isNULL) == length(eType))
       eType <- eType[!isNULL]
       with_z <- with_z[!isNULL]
       null_geometries <- paste("Null geometry IDs:", 
         paste(which(isNULL), collapse=", "))
     }        
-    eType[eType == 5L] <- 2L
-    eType[eType == 6L] <- 3L
+    if (!all_NULL) {
+      eType[eType == 5L] <- 2L
+      eType[eType == 6L] <- 3L
 
-    u_eType <- unique(sort(eType))
-    u_with_z <- unique(sort(with_z))
-    if (length(u_with_z) != 1L) stop(
-      paste("Multiple # dimensions:", 
-        paste((u_with_z + 2), collapse=":")))
-    if (u_with_z < 0 || u_with_z > 1) stop(
-      paste("Invalid # dimensions:", (u_with_z + 2)))
+      u_eType <- unique(sort(eType))
+      u_with_z <- unique(sort(with_z))
+      if (length(u_with_z) != 1L) stop(
+        paste("Multiple # dimensions:", 
+          paste((u_with_z + 2), collapse=":")))
+      if (u_with_z < 0 || u_with_z > 1) stop(
+        paste("Invalid # dimensions:", (u_with_z + 2)))
 
-    t_eType <- table(eType)
-    if (is.null(require_geomType)) {
-      keepGeoms <- NULL
-      if (length(u_eType) > 1L) stop(
-        paste("Multiple incompatible geometries:", 
-          paste(paste(WKB[as.integer(names(t_eType))], t_eType, sep=": "),
-          collapse="; ")))
+      t_eType <- table(eType)
+      if (is.null(require_geomType)) {
+        keepGeoms <- NULL
+        if (length(u_eType) > 1L) stop(
+          paste("Multiple incompatible geometries:", 
+            paste(paste(WKB[as.integer(names(t_eType))], t_eType, sep=": "),
+            collapse="; ")))
 #  if (length(u_eType) == 2L) {
 #    if (u_eType[1] == 2 && u_eType[2] == 5) u_eType = 2
 #    else if (u_eType[1] == 3 && u_eType[2] == 6) u_eType = 3
@@ -110,26 +119,39 @@ ogrInfo <- function(dsn, layer, encoding=NULL, input_field_name_encoding=NULL,
 #      paste(paste(WKB[as.integer(names(t_eType))], t_eType, sep=": "),
 #        collapse="; ")))
 #   }
+      } else {
+        if (!require_geomType %in% WKB[as.integer(names(t_eType))])
+          stop(require_geomType, "not in", WKB[as.integer(names(t_eType))])
+        u_eType <- match(require_geomType, WKB)
+        keepGeoms <- WKB[eType] == require_geomType
+        message("NOTE: keeping only ", sum(keepGeoms), " ", require_geomType,
+          " of ", length(keepGeoms), " features\n",
+          "    note that extent applies to all features")
+      }
     } else {
-      if (!require_geomType %in% WKB[as.integer(names(t_eType))])
-        stop(require_geomType, "not in", WKB[as.integer(names(t_eType))])
-      u_eType <- match(require_geomType, WKB)
-      keepGeoms <- WKB[eType] == require_geomType
-      message("NOTE: keeping only ", sum(keepGeoms), " ", require_geomType,
-        " of ", length(keepGeoms), " features\n",
-        "    note that extent applies to all features")
+      have_features <- FALSE
+      warning("ogrInfo: all features NULL")
     }
   }
+  } else {
+    ogrinfo[[1]] <- attr(ogrinfo[[1]], "dFIDs")
+    warning("ogrInfo: feature count overflow")
+  }
   names(ogrinfo) <- c("nrows", "nitems", "iteminfo", "driver", "extent",
     "nListFields")
   if (ogrinfo$driver == "ESRI Shapefile") {
       DSN <- dsn
       if (!file.info(DSN)$isdir) DSN <- dirname(normalizePath(dsn))
-      con <- file(paste(DSN, .Platform$file.sep, layer, ".dbf", sep=""), "rb")
-      vr <- readBin(con, "raw", n=32L)
-      ldid <- as.integer(vr[30])
-      attr(ogrinfo, "LDID") <- ldid
-      close(con)
+      DBF_fn <- paste(DSN, .Platform$file.sep, layer, ".dbf", sep = "")
+      if (file.exists(DBF_fn)) {
+        con <- file(DBF_fn, "rb")
+        vr <- readBin(con, "raw", n=32L)
+        ldid <- as.integer(vr[30])
+        attr(ogrinfo, "LDID") <- ldid
+        close(con)
+      } else {
+        warning("ogrInfo: ", DBF_fn, " not found", sep="")
+      }
   }
   names(ogrinfo$iteminfo) <- c("name","type","length","typeName","maxListCount")
   if (use_iconv && !is.null(encoding))
@@ -152,7 +174,7 @@ ogrInfo <- function(dsn, layer, encoding=NULL, input_field_name_encoding=NULL,
 print.ogrinfo <- function(x, ...) {
   cat("Source: \"", x$dsn, '\", layer: \"', x$layer, "\"", '\n', sep='')
   cat("Driver:", x$driver)
-  if (x$have_features) cat(" number of rows", x$nrows, "\n")
+  if (x$have_features) cat("; number of rows:", x$nrows, "\n")
   WKB <- c("wkbPoint", "wkbLineString", "wkbPolygon", "wkbMultiPoint",
     "wkbMultiLineString", "wkbMultiPolygon", "wkbGeometryCollection")
   if (!is.null(attr(x, "require_geomType"))) {
@@ -191,8 +213,16 @@ ogrFIDs <- function(dsn, layer){
 }
 
 ogrDrivers <- function() {
-  res <- .Call("ogr_GetDriverNames", PACKAGE="rgdal")
-  res <- as.data.frame(res)
+  if (strsplit(getGDALVersionInfo(), " ")[[1]][2] < "2") {
+    res <- .Call("ogr_GetDriverNames", PACKAGE="rgdal")
+    res <- as.data.frame(res, stringsAsFactors=FALSE)
+  } else {
+      res <- .Call('RGDAL_GetDriverNames', PACKAGE="rgdal")
+      if (!is.null(attr(res, "isVector"))) res$isVector <- attr(res, "isVector")
+      res <- as.data.frame(res, stringsAsFactors=FALSE)
+      res <- res[res$isVector,]
+      names(res)[3] <- "write"
+  }
   res <- res[order(res$name),]
   row.names(res) <- NULL
   res
